(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('@angular/forms'), require('moment')) :
    typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common', '@angular/forms', 'moment'], factory) :
    (factory((global.ng2 = global.ng2 || {}, global.ng2.datepicker = global.ng2.datepicker || {}),global.ng.core,global.ng.common,global.ng.forms,global.moment));
}(this, (function (exports,_angular_core,_angular_common,_angular_forms,moment) { 'use strict';

moment = 'default' in moment ? moment['default'] : moment;

(function (DateState) {
    DateState[DateState["disabled"] = 0] = "disabled";
    DateState[DateState["enabled"] = 1] = "enabled";
    DateState[DateState["inRange"] = 2] = "inRange";
    DateState[DateState["selected"] = 3] = "selected"; //active is also selected and enabled
})(exports.DateState || (exports.DateState = {}));

var BaseSelect = (function () {
    function BaseSelect() {
        this._value = this.EMPTY_VALUE;
        /*@Output()*/
        this.onDateChange = new _angular_core.EventEmitter();
        this.onStateChange = function () { };
        /* we're kinda doing our own change detection because some inputs
         * may be changed directly by template and not through property binding.
         * Such change are not covered by Angular change detection.
         *
         * We just want to know if something has changed not what has changed so
         * a boolean flag is enough
         */
        this.hasStateChanged = false;
    }
    /**
     * Extend the base configuration needed by @Directive
     * @param {Directive} config           subclass configuration
     * @param {Function}  directiveClasses subclass
     * @param {any[]}     ...a             useless just to please compiler if subclass wants to add parameter
     */
    //TODO the ...a trick works to keep compiler quiet but this will be transpiled into unseless code
    BaseSelect.extendConfig = function (config, directiveClasses) {
        var a = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            a[_i - 2] = arguments[_i];
        }
        return extendDirConfig({
            //we could auto-generate it using gulp or something
            inputs: ['minDate', 'maxDate'],
            outputs: ['onDateChange'],
            providers: [selectProvider(directiveClasses)],
        }, config);
    };
    Object.defineProperty(BaseSelect.prototype, "EMPTY_VALUE", {
        get: function () { },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseSelect.prototype, "value", {
        get: function () {
            return this._value;
        },
        /**
         * Set the value without any check (except null) and emit an onDateChange event
         * @param {T} value
         */
        set: function (value) {
            /* Use of a function so it can be override */
            this._setValue(value);
        },
        enumerable: true,
        configurable: true
    });
    // TODO add an emitEvent parameter ? We would have to delete the setter
    BaseSelect.prototype._setValue = function (value) {
        if (value !== this._value) {
            this.onDateChange.emit(this._value = value || this.EMPTY_VALUE);
            this.hasStateChanged = true;
        }
    };
    Object.defineProperty(BaseSelect.prototype, "minDate", {
        /*@Input()*/
        get: function () {
            return this._minDate;
        },
        set: function (date) {
            this._minDate = date;
            this.hasStateChanged = true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(BaseSelect.prototype, "maxDate", {
        /*@Input()*/
        get: function () {
            return this._maxDate;
        },
        set: function (date) {
            this._maxDate = date;
            this.hasStateChanged = true;
        },
        enumerable: true,
        configurable: true
    });
    BaseSelect.prototype.registerOnStateChange = function (fn) {
        this.onStateChange = fn;
    };
    BaseSelect.prototype.ngDoCheck = function () {
        if (this.hasStateChanged)
            this.onStateChange();
        this.hasStateChanged = false;
    };
    /**
     * Return a date corresponding to the day of the input date.
     * We don't want to introduce time in our dates.
     * @param  {moment.Moment} date A date that may have time in it
     * @return {moment.Moment}      A date corresponding to a day with all time unit to 0.
     */
    BaseSelect.prototype.getDay = function (date) {
        return date ? moment([date.year, date.month, date.date]) : null;
    };
    BaseSelect.prototype.isDateInSelectRange = function (date) {
        return this.isDateSelected(date);
    };
    /** Returns true when date is between minDate and maxDate */
    BaseSelect.prototype.isDateValid = function (date) {
        return date &&
            (!this.minDate || date.isSameOrAfter(this.minDate, 'd')) &&
            (!this.maxDate || date.isSameOrBefore(this.maxDate, 'd'));
    };
    /**
     * return true if date is selectable meaning :
     *  - not null
     *  - valid
     *  - not already selected
     * Primarly meant to be used by selectDate() of subclass
     * @param  {moment.Moment} date [description]
     * @return {boolean}            [description]
     */
    BaseSelect.prototype.isDateSelectable = function (date) {
        return date && this.isDateValid(date) && !this.isDateSelected(date);
    };
    BaseSelect.prototype.getDateState = function (date) {
        if (this.isDateSelected(date))
            return exports.DateState.selected;
        if (this.isDateInSelectRange(date))
            return exports.DateState.inRange;
        if (this.isDateValid(date))
            return exports.DateState.enabled;
        return exports.DateState.disabled;
    };
    return BaseSelect;
}());
//helper
function isSameDay(date1, date2) {
    if (date1 && date2)
        return date1.isSame(date2, 'd');
    return false;
}

function concatArr(defaultArr, newArr) {
    return defaultArr ?
        (newArr ? defaultArr.concat(newArr) : defaultArr) :
        newArr;
}
function extendDirConfig(defaultConfig, config) {
    return {
        providers: concatArr(defaultConfig.providers, config.providers),
        inputs: concatArr(defaultConfig.inputs, config.inputs),
        outputs: concatArr(defaultConfig.outputs, config.outputs),
        queries: Object.assign({}, defaultConfig.queries, config.queries),
        host: Object.assign({}, defaultConfig.host, config.host),
        selector: config.selector || defaultConfig.selector,
        exportAs: config.exportAs || defaultConfig.exportAs,
    };
}
function extendConfig(defaultConfig, config) {
    return Object.assign(extendDirConfig(defaultConfig, config), {
        template: (config.template || config.templateUrl) ? config.template : defaultConfig.template,
        templateUrl: (config.template || config.templateUrl) ? config.templateUrl : defaultConfig.templateUrl,
        styles: (config.styles || config.styleUrls) ? config.styles : defaultConfig.styles,
        styleUrls: (config.styles || config.styleUrls) ? config.styleUrls : defaultConfig.styleUrls,
        moduleId: (config.templateUrl || config.styleUrls) ? config.moduleId :
            ((!config.template && defaultConfig.templateUrl) || (!config.styles && defaultConfig.styleUrls) ?
                defaultConfig.moduleId : undefined),
        viewProviders: concatArr(defaultConfig.viewProviders, config.viewProviders),
        animations: concatArr(defaultConfig.animations, config.animations),
        entryComponents: concatArr(defaultConfig.entryComponents, config.entryComponents),
        changeDetection: config.changeDetection || defaultConfig.changeDetection,
        encapsulation: config.encapsulation || defaultConfig.encapsulation,
        interpolation: config.interpolation || defaultConfig.interpolation
    });
}
function formProvider(componentClass) {
    return {
        provide: _angular_forms.NG_VALUE_ACCESSOR,
        useExisting: _angular_core.forwardRef(function () { return componentClass; }),
        multi: true
    };
}
function selectProvider(directiveClass) {
    return {
        provide: BaseSelect, useExisting: _angular_core.forwardRef(function () { return directiveClass; })
    };
}

var DatePickerTemplate = (function () {
    function DatePickerTemplate(cd, select) {
        var _this = this;
        this.cd = cd;
        this.select = select;
        /*@Input()*/ this.showSixWeek = false;
        this.weekDaysName = moment.weekdaysShort(true);
        /**
         * /!\ Only use it on template or as a READONLY property. Should never be mutated, use helpers functions instead.
         * It's public because of aot.
         * @type {Month[]}
         */
        this.months = [];
        /* Value accessor stuff */
        this.onTouchedCallback = function () { };
        this.onChangeCallback = function () { };
        if (!select)
            throw 'No SelectDirective specified. DatePicker must be coupled with a SelectDirective';
        this.select.registerOnStateChange(function () {
            //Do it on next tick to avoid detached element to be processed
            setTimeout(function () {
                _this.updateCalendarDays();
            });
        });
        // should we unsubscribe onDestroy since SelectDirective has
        // same lifecycle that this component ?
        this.select.onDateChange.subscribe(function (d) {
            _this.onChangeCallback(d);
        });
    }
    /**
    * Extend the base configuration needed by @Component
    * @param {Component} config           subclass configuration
    * @param {any[]}     ...a             useless just to please compiler if subclass wants to add parameter
    */
    //TODO the ...a trick works to keep compiler quiet but this will be transpiled into unseless code
    DatePickerTemplate.extendConfig = function (config, componentClass) {
        var a = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            a[_i - 2] = arguments[_i];
        }
        return extendConfig({
            //we could auto-generate it using gulp or something
            inputs: ['locale', 'showSixWeek'],
            providers: [formProvider(componentClass)],
            changeDetection: _angular_core.ChangeDetectionStrategy.OnPush
        }, config);
    };
    Object.defineProperty(DatePickerTemplate.prototype, "locale", {
        get: function () {
            return this._locale;
        },
        /*@Input()*/ set: function (locale) {
            this._locale = (!locale || locale.length == 0) ? false : locale;
            //with global locale we can call weekdaysShort() with a boolean to get array in locale order
            //unfortunately we can't for local locale, must do it ourselves.
            var localeData = this.applyLocale(moment()).localeData();
            var weekdays = localeData.weekdaysShort();
            // typings not up to date
            this.weekDaysName = weekdays.concat(weekdays.splice(0, localeData.firstDayOfWeek()));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DatePickerTemplate.prototype, "month", {
        /**
         * helper function when displaying only 1 month.
         * /!\ Like {@link #months} it's meant for use on template only. Never mutate this property.
         * @return {Month} [description]
         */
        get: function () {
            return this.months[0];
        },
        enumerable: true,
        configurable: true
    });
    // TODO should we check that value match SelectDirective's expected value type ?
    // If so how ?
    // TODO use setValue() ?
    DatePickerTemplate.prototype.writeValue = function (value) {
        this.select.value = value;
    };
    DatePickerTemplate.prototype.registerOnChange = function (fn) {
        this.onChangeCallback = fn;
    };
    DatePickerTemplate.prototype.registerOnTouched = function (fn) {
        this.onTouchedCallback = fn;
    };
    Object.defineProperty(DatePickerTemplate.prototype, "value", {
        /* */
        //helper
        get: function () {
            return this.select.value;
        },
        set: function (v) {
            this.select.value = v;
        },
        enumerable: true,
        configurable: true
    });
    DatePickerTemplate.prototype.applyLocale = function (date) {
        if (this.locale)
            date.locale(this.locale);
        return date;
    };
    /**
     * Generate an Array of CalendarDay representing a month to display,
     * with extra days from previous and next month to get plain weeks.
     * @param  {number}        month
     * @param  {number}        year
     * @param  {boolean}       showSixWeek true if we should always returns 6 weeks,
     * this will avoid calendar to change size depending on month displayed.
     * @return {CalendarDay[]}             Array of CalendarDay representing a month to display
     */
    DatePickerTemplate.prototype.generateCalendarDays = function (date) {
        var today = moment();
        //start date
        var itDate = this.applyLocale(date.clone());
        itDate.subtract(itDate.weekday(), 'd');
        //end date
        var endDate = this.applyLocale(date.clone().endOf('month'));
        endDate.add(6 - endDate.weekday(), 'd');
        if (this.showSixWeek) {
            var nbWeeks = endDate.diff(itDate, 'weeks');
            if (nbWeeks < 5)
                endDate.add(5 - nbWeeks, 'weeks');
        }
        var days = [];
        while (itDate.isBefore(endDate)) {
            days.push({
                date: itDate.clone(),
                state: this.select.getDateState(itDate),
                isToday: today.isSame(itDate, 'day'),
                isCurrDisplayMonth: itDate.month() == date.month()
            });
            itDate = itDate.add(1, 'd');
        }
        return days;
    };
    DatePickerTemplate.prototype.updateCalendarDays = function () {
        for (var _i = 0, _a = this.months; _i < _a.length; _i++) {
            var m = _a[_i];
            var days = m.days;
            for (var i = 0, l = days.length; i < l; i++) {
                var day = days[i], state = this.select.getDateState(day.date);
                if (day.state != state) {
                    days[i] = {
                        date: day.date,
                        isToday: day.isToday,
                        isCurrDisplayMonth: day.isCurrDisplayMonth,
                        state: state
                    };
                }
            }
        }
        this.cd.markForCheck();
    };
    DatePickerTemplate.prototype.newMonth = function (date) {
        if (date === void 0) { date = moment(); }
        var monthDate = moment([date.year(), date.month()]);
        return {
            date: monthDate,
            days: this.generateCalendarDays(monthDate)
        };
    };
    DatePickerTemplate.prototype.initMonths = function () {
        var _this = this;
        var dates = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            dates[_i - 0] = arguments[_i];
        }
        this.months = dates.map(function (d) { return _this.newMonth(d); });
        this.cd.markForCheck();
    };
    DatePickerTemplate.prototype.getMonthDate = function (idx) {
        if (idx === void 0) { idx = 0; }
        var month = this.months[idx];
        if (month)
            return month.date.clone();
        return null;
    };
    DatePickerTemplate.prototype.addMonth = function (date) {
        if (date === void 0) { date = moment(); }
        if (date) {
            this.months.push(this.newMonth(date));
            this.cd.markForCheck();
            return this.months.length - 1;
        }
        return -1;
    };
    DatePickerTemplate.prototype.removeMonth = function (idx) {
        this.months.splice(idx, 1);
        this.cd.markForCheck();
    };
    DatePickerTemplate.prototype.changeMonth = function (date, idx) {
        if (idx === void 0) { idx = 0; }
        var current_month = this.months[idx];
        if (date && !date.isSame(current_month.date, 'M')) {
            var monthDate = moment([date.year(), date.month()]);
            current_month.date = monthDate;
            current_month.days = this.generateCalendarDays(monthDate);
            this.cd.markForCheck();
        }
    };
    return DatePickerTemplate;
}());

var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate$1 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$1 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var DatePickerIonicComponent = (function (_super) {
    __extends(DatePickerIonicComponent, _super);
    function DatePickerIonicComponent(renderer, elRef, cd, select) {
        var _this = this;
        _super.call(this, cd, select);
        this.renderer = renderer;
        this._class = DatePickerIonicComponent.CONTAINER_CLASS;
        this._opened = false;
        /*@Input()*/ this.viewFormat = 'LL';
        this.el = elRef.nativeElement;
        renderer.listenGlobal('window', 'click', function (e) {
            if (_this.opened &&
                e.target &&
                _this.el !== e.target &&
                !_this.el.contains(e.target))
                _this.close();
        });
    }
    DatePickerIonicComponent.extendConfig = function (config, componentClass) {
        var a = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            a[_i - 2] = arguments[_i];
        }
        return extendConfig(_super.extendConfig.call(this, {
            template: "<div [class]=\"class\"><input type=\"hidden\" [(ngModel)]=\"value\"><div class=\"ui-kit-calendar-input\" [class.opened]=\"opened || expanded\" (click)=\"inputClick()\"><span [class.opened]=\"opened || expanded\">{{ select.toString(viewFormat, locale) }}</span><i class=\"ion-ios-calendar-outline\"></i></div><div class=\"ui-kit-calendar-cal-container\" [class.opened]=\"opened\"><div class=\"ui-kit-calendar-cal-top\"><i class=\"ion-arrow-left-b\" (click)=\"prevMonth()\"></i><span>{{ month.date | moment: 'MMMM YYYY':locale }}</span><i class=\"ion-arrow-right-b\" (click)=\"nextMonth()\"></i></div><div class=\"ui-kit-calendar-day-names\"><span *ngFor=\"let wd of weekDaysName\">{{ wd }}</span></div><div class=\"ui-kit-calendar-days\"><span *ngFor=\"let d of month.days\"(click)=\"onDateclick($event, d)\"[ngClass]=\"d | dayClasses\">{{ d.date | moment: 'D' }}</span></div></div></div>",
            styles: [".ui-kit-calendar-container { position: relative; width: 170px; margin: 0 10px 0 0; display: block; float: left; position: relative; z-index: 99; } .ui-kit-calendar-container .ui-kit-calendar-input { border: 1px solid #aec9de; border-radius: 10px; display: -ms-flexbox; display: flex; -ms-flex-pack: center; justify-content: center; -ms-flex-align: center; align-items: center; width: 70px; height: 40px; background: #fcfcff; cursor: pointer; text-align: center; font-size: 0; transition: all 300ms ease; } .ui-kit-calendar-container .ui-kit-calendar-input i { color: #505b71; font-size: 20px; } .ui-kit-calendar-container .ui-kit-calendar-input.opened { width: 100%; border: 1px solid #44c8f9; color: #44c8f9; background: rgba(68, 200, 249, 0.15); font-size: 12px; } .ui-kit-calendar-container .ui-kit-calendar-input.opened i { color: #44c8f9; } .ui-kit-calendar-container .ui-kit-calendar-input span { display: none; text-overflow: ellipsis; overflow: hidden; white-space: nowrap; } .ui-kit-calendar-container .ui-kit-calendar-input span.opened { display: inline; margin-right: 10px; } .ui-kit-calendar-container .ui-kit-calendar-cal-container { position: absolute; top: 55px; right: 0; color: #fff; background: #44c8f9; display: inline-block; width: 300px; border-radius: 3px; visibility: hidden; opacity: 0; transition: visibility 0s, opacity 300ms linear 300ms; } .ui-kit-calendar-container .ui-kit-calendar-cal-container.opened { visibility: visible; opacity: 1; } .ui-kit-calendar-container .ui-kit-calendar-cal-container:after { position: absolute; top: -15px; right: 28px; content: \"\"; width: 0; height: 0; border-top: 15px solid transparent; border-bottom: 15px solid transparent; border-left: 15px solid #44c8f9; transform: rotate(-90deg); } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-cal-top { width: 100%; height: 40px; line-height: 40px; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-cal-top i { display: block; float: left; font-size: 16px; width: 40px; height: 40px; text-align: center; cursor: pointer; vertical-align: baseline; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-cal-top span { display: block; float: left; width: 220px; height: 40px; font-size: 14px; text-align: center; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-day-names { width: 100%; height: 23px; background: #35b1e2; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-day-names span { font-size: 10px; line-height: 23px; text-align: center; display: block; float: left; width: calc(100% / 7); height: 23px; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-days { width: 100%; padding: 5px 0; margin-bottom: 15px; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-days span { font-size: 12px; display: block; float: left; text-align: center; height: 40px; line-height: 40px; width: calc(100% / 7); cursor: pointer; border-radius: 50%; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-days span.today { border: 1px solid #fff; box-sizing: border-box; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-days span:not(.currDisplayMonth) { opacity: 0; pointer-events: none; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-days span.disabled { opacity: 0.5; pointer-events: none; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-days span.inRange { background: rgba(255, 255, 255, 0.5); color: #44c8f9; } .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-days span.enabled:hover, .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-days span.inRange:hover, .ui-kit-calendar-container .ui-kit-calendar-cal-container .ui-kit-calendar-days span.selected { background: #fff; color: #44c8f9; } /*# sourceMappingURL=ionic.css.map */ "],
            inputs: ['class', 'expanded', 'opened', 'viewFormat']
        }, componentClass), config);
    };
    Object.defineProperty(DatePickerIonicComponent.prototype, "class", {
        /*@Input()*/
        get: function () {
            return this._class;
        },
        set: function (classStr) {
            this._class = DatePickerIonicComponent.CONTAINER_CLASS + ' ' + classStr;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DatePickerIonicComponent.prototype, "opened", {
        get: function () {
            return this._opened;
        },
        /*@Input()*/
        set: function (b) {
            if (this._opened !== b) {
                this._opened = b;
                this.cd.markForCheck();
            }
        },
        enumerable: true,
        configurable: true
    });
    DatePickerIonicComponent.prototype.ngOnInit = function () {
        this.initMonths(moment());
    };
    DatePickerIonicComponent.prototype.onDateclick = function (e, day) {
        e.preventDefault();
        this.select.selectDate(day.date);
        if (this.select.isComplete())
            this.close();
    };
    DatePickerIonicComponent.prototype.nextMonth = function () {
        this.changeMonth(this.getMonthDate().add(1, 'M'));
    };
    DatePickerIonicComponent.prototype.prevMonth = function () {
        this.changeMonth(this.getMonthDate().subtract(1, 'M'));
    };
    DatePickerIonicComponent.prototype.toggle = function () {
        this.opened = !this.opened;
    };
    DatePickerIonicComponent.prototype.open = function () {
        this.opened = true;
    };
    DatePickerIonicComponent.prototype.close = function () {
        this.opened = false;
    };
    DatePickerIonicComponent.prototype.inputClick = function () {
        this.onTouchedCallback();
        this.toggle();
    };
    DatePickerIonicComponent.CONTAINER_CLASS = 'ui-kit-calendar-container';
    DatePickerIonicComponent = __decorate$1([
        _angular_core.Component(DatePickerIonicComponent.extendConfig({
            selector: 'datepicker-ionic'
        }, DatePickerIonicComponent)),
        __param(3, _angular_core.Optional()), 
        __metadata$1('design:paramtypes', [_angular_core.Renderer, _angular_core.ElementRef, _angular_core.ChangeDetectorRef, BaseSelect])
    ], DatePickerIonicComponent);
    return DatePickerIonicComponent;
}(DatePickerTemplate));

var __decorate$3 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$3 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var MomentPipe = (function () {
    function MomentPipe() {
    }
    MomentPipe.prototype.transform = function (date, format, locale) {
        if (format === void 0) { format = 'LL'; }
        if (!date)
            return '';
        var d = date;
        if (locale) {
            d = moment(d);
            d.locale(locale);
        }
        return d.format(format);
    };
    MomentPipe = __decorate$3([
        _angular_core.Pipe({
            name: 'moment'
        }), 
        __metadata$3('design:paramtypes', [])
    ], MomentPipe);
    return MomentPipe;
}());

var __extends$1 = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate$2 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$2 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var SingleSelectDirective = (function (_super) {
    __extends$1(SingleSelectDirective, _super);
    function SingleSelectDirective(momentPipe) {
        _super.call(this);
        this.momentPipe = momentPipe;
    }
    Object.defineProperty(SingleSelectDirective.prototype, "EMPTY_VALUE", {
        get: function () { return null; },
        enumerable: true,
        configurable: true
    });
    
    SingleSelectDirective.prototype.setValue = function (value) {
        if (!value || this.isDateValid(value))
            this._setValue(value);
    };
    SingleSelectDirective.prototype.ngOnChanges = function (changes) {
        if (!this.isDateValid(this.value))
            this._setValue(null);
    };
    SingleSelectDirective.prototype.selectDate = function (date) {
        if (!this.isDateSelectable(date))
            return false;
        this._setValue(date);
        return true;
    };
    SingleSelectDirective.prototype.unselectDate = function (date) {
        if (this.isDateSelected(date)) {
            this._setValue(null);
            return true;
        }
        return false;
    };
    SingleSelectDirective.prototype.isDateSelected = function (date) {
        return isSameDay(date, this.value);
    };
    SingleSelectDirective.prototype.isComplete = function () {
        return !!this.value;
    };
    SingleSelectDirective.prototype.toString = function (format, locale) {
        if (format === void 0) { format = 'LL'; }
        return this.momentPipe.transform(this.value, format, locale);
    };
    SingleSelectDirective = __decorate$2([
        _angular_core.Directive(BaseSelect.extendConfig({
            selector: '[singleSelect]'
        }, SingleSelectDirective)), 
        __metadata$2('design:paramtypes', [MomentPipe])
    ], SingleSelectDirective);
    return SingleSelectDirective;
}(BaseSelect));

var __extends$2 = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate$4 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$4 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var MultiSelectDirective = (function (_super) {
    __extends$2(MultiSelectDirective, _super);
    function MultiSelectDirective(momentPipe) {
        _super.call(this);
        this.momentPipe = momentPipe;
        this._limit = Infinity;
    }
    Object.defineProperty(MultiSelectDirective.prototype, "EMPTY_VALUE", {
        get: function () {
            return [];
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MultiSelectDirective.prototype, "limit", {
        get: function () {
            return this._limit;
        },
        set: function (limit) {
            this._limit = limit;
        },
        enumerable: true,
        configurable: true
    });
    MultiSelectDirective.prototype.setValue = function (dates) {
        var _this = this;
        if (dates != this.value) {
            this.value = (dates || [])
                .filter(function (d) { return _this.isDateValid(d); })
                .splice(0, dates.length - this.limit)
                .map(function (d) { return _this.getDay(d); })
                .sort(function (a, b) { return a - b; });
        }
    };
    MultiSelectDirective.prototype.ngOnChanges = function (changes) {
        var _this = this;
        var value = this.value;
        if (changes['minDate'] || changes['maxDate']) {
            var newValue = value.filter(function (d) { return _this.isDateValid(d); });
            if (newValue.length < value.length)
                value = newValue;
        }
        if (changes['limit'] && value.length > this.limit)
            value = value.slice(0, this.limit);
        this.value = value;
    };
    MultiSelectDirective.prototype.selectDate = function (date) {
        if (!this.isDateSelectable(date))
            return false;
        if (this.value.length < this.limit)
            this.value = this.value.concat([date])
                .sort(function (a, b) { return a - b; });
        else
            this.value = [date];
    };
    MultiSelectDirective.prototype.unselectDate = function (date) {
        var newValue = this.value.filter(function (d) { return !isSameDay(d, date); });
        if (newValue.length < this.value.length) {
            this.value = newValue;
            return true;
        }
        return false;
    };
    MultiSelectDirective.prototype.isDateSelected = function (date) {
        return !!this.value.find(function (d) { return isSameDay(d, date); });
    };
    /*
    isDateInSelectRange(date: moment.Moment): boolean {
      return this.value.length > 1 &&
            date.isBetween(this.value[0], this.value[this.value.length - 1], 'day', '[]');
    }
    */
    MultiSelectDirective.prototype.isComplete = function () {
        return this.value && this.value.length == this.limit;
    };
    MultiSelectDirective.prototype.toString = function (format, locale) {
        var _this = this;
        if (format === void 0) { format = 'LL'; }
        return this.value.reduce(function (prev, date, idx) {
            return prev + (idx == 0 ? '' : ' - ') + _this.momentPipe.transform(date, format, locale);
        }, '');
    };
    __decorate$4([
        _angular_core.Input('multiSelect'), 
        __metadata$4('design:type', Object)
    ], MultiSelectDirective.prototype, "limit", null);
    MultiSelectDirective = __decorate$4([
        _angular_core.Directive(BaseSelect.extendConfig({
            selector: '[multiSelect]'
        }, MultiSelectDirective)), 
        __metadata$4('design:paramtypes', [MomentPipe])
    ], MultiSelectDirective);
    return MultiSelectDirective;
}(BaseSelect));

var __extends$3 = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate$5 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$5 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var RangeSelectDirective = (function (_super) {
    __extends$3(RangeSelectDirective, _super);
    function RangeSelectDirective(momentPipe) {
        _super.call(this);
        this.momentPipe = momentPipe;
    }
    Object.defineProperty(RangeSelectDirective.prototype, "EMPTY_VALUE", {
        get: function () {
            return {
                start: null,
                end: null
            };
        },
        enumerable: true,
        configurable: true
    });
    
    RangeSelectDirective.prototype.setValue = function (value) {
        if (!value)
            this.value = null;
        else if (value !== this.value) {
            if (!this.isDateValid(value.start))
                value.start = null;
            if (!this.isDateValid(value.end))
                value.end = null;
            this.value = value;
        }
    };
    RangeSelectDirective.prototype.setStartDate = function (date) {
        this.value = {
            start: date,
            end: this.value.end
        };
    };
    RangeSelectDirective.prototype.setEndDate = function (date) {
        this.value = {
            start: this.value.start,
            end: date
        };
    };
    RangeSelectDirective.prototype.ngOnChanges = function (changes) {
        var start = this.value.start, end = this.value.end;
        if (!this.isDateValid(this.value.start))
            start = null;
        if (!this.isDateValid(this.value.end))
            end = null;
        if (start !== this.value.start || end !== this.value.end)
            this.value = {
                start: start,
                end: end
            };
    };
    RangeSelectDirective.prototype.selectDate = function (date) {
        if (!date || !this.isDateValid(date))
            return false;
        if (this.isDateSelected(date))
            return !this.unselectDate(date);
        var start = this.value.start, end = this.value.end;
        if (!start)
            start = date;
        else if (!end)
            end = date;
        else {
            if (Math.abs(date.diff(start)) <= Math.abs(date.diff(end)))
                this.setStartDate(date);
            else
                this.setEndDate(date);
            return true;
        }
        if (start && end && start.isAfter(end)) {
            this.value = {
                start: end,
                end: start
            };
        }
        else
            this.value = {
                start: start,
                end: end
            };
        return true;
    };
    RangeSelectDirective.prototype.unselectDate = function (date) {
        if (date) {
            if (isSameDay(date, this.value.start)) {
                this.setStartDate(null);
                return true;
            }
            if (isSameDay(date, this.value.end)) {
                this.setEndDate(null);
                return true;
            }
        }
        return false;
    };
    RangeSelectDirective.prototype.isDateSelected = function (date) {
        return date && (isSameDay(date, this.value.start) || isSameDay(date, this.value.end));
    };
    RangeSelectDirective.prototype.isDateInSelectRange = function (date) {
        var start = this.value.start, end = this.value.end;
        return start && end &&
            date.isSameOrAfter(start, 'd') &&
            date.isSameOrBefore(end, 'd');
    };
    RangeSelectDirective.prototype.isComplete = function () {
        return !!this.value.start && !!this.value.end;
    };
    RangeSelectDirective.prototype.toString = function (format, locale) {
        if (format === void 0) { format = 'LL'; }
        if (!this.value || (!this.value.start && !this.value.end))
            return '';
        return this.momentPipe.transform(this.value.start, format, locale) +
            ' - ' +
            this.momentPipe.transform(this.value.end, format, locale);
    };
    RangeSelectDirective = __decorate$5([
        _angular_core.Directive(BaseSelect.extendConfig({
            selector: '[rangeSelect]'
        }, RangeSelectDirective)), 
        __metadata$5('design:paramtypes', [MomentPipe])
    ], RangeSelectDirective);
    return RangeSelectDirective;
}(BaseSelect));

var __decorate$6 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$6 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var EqualMomentPipe = (function () {
    function EqualMomentPipe() {
    }
    EqualMomentPipe.prototype.transform = function (value, compare, unit) {
        if (unit === void 0) { unit = 'd'; }
        if (!value || !compare)
            return !value && !compare;
        //TODO bad types, weird
        return value.isSame(compare, unit);
    };
    EqualMomentPipe = __decorate$6([
        _angular_core.Pipe({
            name: 'eqMoment'
        }), 
        __metadata$6('design:paramtypes', [])
    ], EqualMomentPipe);
    return EqualMomentPipe;
}());

var __decorate$7 = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$7 = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var DayClassesPipe = (function () {
    function DayClassesPipe() {
    }
    DayClassesPipe.prototype.transform = function (day) {
        var classes = {
            disabled: false,
            enabled: false,
            inRange: false,
            selected: false,
            today: day.isToday,
            currDisplayMonth: day.isCurrDisplayMonth
        };
        if (day.state !== undefined && day.state !== null)
            classes[exports.DateState[day.state]] = true;
        return classes;
    };
    DayClassesPipe = __decorate$7([
        _angular_core.Pipe({
            name: 'dayClasses'
        }), 
        __metadata$7('design:paramtypes', [])
    ], DayClassesPipe);
    return DayClassesPipe;
}());

var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var decl_exports = [
    DatePickerIonicComponent,
    SingleSelectDirective,
    MultiSelectDirective,
    RangeSelectDirective,
    MomentPipe,
    EqualMomentPipe,
    DayClassesPipe
];
var DatePickerModule = (function () {
    function DatePickerModule() {
    }
    DatePickerModule = __decorate([
        _angular_core.NgModule({
            imports: [
                _angular_common.CommonModule,
                _angular_forms.FormsModule
            ],
            declarations: decl_exports,
            exports: decl_exports,
            providers: [{
                    provide: MomentPipe,
                    useClass: MomentPipe
                }]
        }), 
        __metadata('design:paramtypes', [])
    ], DatePickerModule);
    return DatePickerModule;
}());

exports.DatePickerModule = DatePickerModule;
exports.BaseSelect = BaseSelect;
exports.isSameDay = isSameDay;
exports.SingleSelectDirective = SingleSelectDirective;
exports.MultiSelectDirective = MultiSelectDirective;
exports.RangeSelectDirective = RangeSelectDirective;
exports.DatePickerTemplate = DatePickerTemplate;
exports.DatePickerIonicComponent = DatePickerIonicComponent;
exports.extendDirConfig = extendDirConfig;
exports.extendConfig = extendConfig;
exports.formProvider = formProvider;
exports.selectProvider = selectProvider;

Object.defineProperty(exports, '__esModule', { value: true });

})));

//# sourceMappingURL=ng2-datepicker.umd.js.map
