import { Component, ChangeDetectorRef } from '@angular/core';
import { ControlValueAccessor } from '@angular/forms';
import moment from 'moment';
import { BaseSelect } from '../selections/base.select';
import { CalendarDay } from '../models';
export interface Month {
    date: moment.Moment;
    days: CalendarDay[];
}
export declare abstract class DatePickerTemplate<T extends BaseSelect<V>, V> implements ControlValueAccessor {
    protected cd: ChangeDetectorRef;
    protected select: T;
    /**
    * Extend the base configuration needed by @Component
    * @param {Component} config           subclass configuration
    * @param {any[]}     ...a             useless just to please compiler if subclass wants to add parameter
    */
    static extendConfig(config: Component, componentClass: Function, ...a: any[]): Component;
    private _locale;
    locale: (string | false);
    showSixWeek: boolean;
    weekDaysName: string[];
    /**
     * /!\ Only use it on template or as a READONLY property. Should never be mutated, use helpers functions instead.
     * It's public because of aot.
     * @type {Month[]}
     */
    months: Month[];
    /**
     * helper function when displaying only 1 month.
     * /!\ Like {@link #months} it's meant for use on template only. Never mutate this property.
     * @return {Month} [description]
     */
    readonly month: Month;
    constructor(cd: ChangeDetectorRef, select: T);
    onTouchedCallback: () => void;
    private onChangeCallback;
    writeValue(value: V): void;
    registerOnChange(fn: (_: V) => void): void;
    registerOnTouched(fn: () => void): void;
    value: V;
    protected applyLocale(date: moment.Moment): moment.Moment;
    /**
     * Generate an Array of CalendarDay representing a month to display,
     * with extra days from previous and next month to get plain weeks.
     * @param  {number}        month
     * @param  {number}        year
     * @param  {boolean}       showSixWeek true if we should always returns 6 weeks,
     * this will avoid calendar to change size depending on month displayed.
     * @return {CalendarDay[]}             Array of CalendarDay representing a month to display
     */
    private generateCalendarDays(date);
    private updateCalendarDays();
    private newMonth(date?);
    initMonths(...dates: moment.Moment[]): void;
    getMonthDate(idx?: number): moment.Moment;
    addMonth(date?: moment.Moment): number;
    removeMonth(idx: number): void;
    changeMonth(date: moment.Moment, idx?: number): void;
}
