var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Directive } from '@angular/core';
import { BaseSelect, isSameDay } from './base.select';
import { MomentPipe } from '../pipes/moment';
export var RangeSelectDirective = (function (_super) {
    __extends(RangeSelectDirective, _super);
    function RangeSelectDirective(momentPipe) {
        _super.call(this);
        this.momentPipe = momentPipe;
    }
    Object.defineProperty(RangeSelectDirective.prototype, "EMPTY_VALUE", {
        get: function () {
            return {
                start: null,
                end: null
            };
        },
        enumerable: true,
        configurable: true
    });
    ;
    RangeSelectDirective.prototype.setValue = function (value) {
        if (!value)
            this.value = null;
        else if (value !== this.value) {
            if (!this.isDateValid(value.start))
                value.start = null;
            if (!this.isDateValid(value.end))
                value.end = null;
            this.value = value;
        }
    };
    RangeSelectDirective.prototype.setStartDate = function (date) {
        this.value = {
            start: date,
            end: this.value.end
        };
    };
    RangeSelectDirective.prototype.setEndDate = function (date) {
        this.value = {
            start: this.value.start,
            end: date
        };
    };
    RangeSelectDirective.prototype.ngOnChanges = function (changes) {
        var start = this.value.start, end = this.value.end;
        if (!this.isDateValid(this.value.start))
            start = null;
        if (!this.isDateValid(this.value.end))
            end = null;
        if (start !== this.value.start || end !== this.value.end)
            this.value = {
                start: start,
                end: end
            };
    };
    RangeSelectDirective.prototype.selectDate = function (date) {
        if (!date || !this.isDateValid(date))
            return false;
        if (this.isDateSelected(date))
            return !this.unselectDate(date);
        var start = this.value.start, end = this.value.end;
        if (!start)
            start = date;
        else if (!end)
            end = date;
        else {
            if (Math.abs(date.diff(start)) <= Math.abs(date.diff(end)))
                this.setStartDate(date);
            else
                this.setEndDate(date);
            return true;
        }
        if (start && end && start.isAfter(end)) {
            this.value = {
                start: end,
                end: start
            };
        }
        else
            this.value = {
                start: start,
                end: end
            };
        return true;
    };
    RangeSelectDirective.prototype.unselectDate = function (date) {
        if (date) {
            if (isSameDay(date, this.value.start)) {
                this.setStartDate(null);
                return true;
            }
            if (isSameDay(date, this.value.end)) {
                this.setEndDate(null);
                return true;
            }
        }
        return false;
    };
    RangeSelectDirective.prototype.isDateSelected = function (date) {
        return date && (isSameDay(date, this.value.start) || isSameDay(date, this.value.end));
    };
    RangeSelectDirective.prototype.isDateInSelectRange = function (date) {
        var start = this.value.start, end = this.value.end;
        return start && end &&
            date.isSameOrAfter(start, 'd') &&
            date.isSameOrBefore(end, 'd');
    };
    RangeSelectDirective.prototype.isComplete = function () {
        return !!this.value.start && !!this.value.end;
    };
    RangeSelectDirective.prototype.toString = function (format, locale) {
        if (format === void 0) { format = 'LL'; }
        if (!this.value || (!this.value.start && !this.value.end))
            return '';
        return this.momentPipe.transform(this.value.start, format, locale) +
            ' - ' +
            this.momentPipe.transform(this.value.end, format, locale);
    };
    RangeSelectDirective = __decorate([
        Directive(BaseSelect.extendConfig({
            selector: '[rangeSelect]'
        }, RangeSelectDirective)), 
        __metadata('design:paramtypes', [MomentPipe])
    ], RangeSelectDirective);
    return RangeSelectDirective;
}(BaseSelect));

//# sourceMappingURL=range.select.js.map
